package roneditor;

import org.eclipse.core.runtime.Assert;
import org.eclipse.emf.common.notify.Adapter;
import org.eclipse.emf.common.notify.Notifier;
import org.eclipse.emf.ecore.EObject;
import org.eclipse.ui.IEditorPart;
import org.eclipse.ui.ISelectionService;
import org.eclipse.ui.IViewPart;
import org.eclipse.ui.IWorkbenchPage;
import org.eclipse.ui.IWorkbenchPart;
import org.eclipse.ui.part.IPage;
import org.eclipse.ui.part.IPageBookViewPage;
import org.eclipse.ui.part.MessagePage;
import org.eclipse.ui.part.PageBook;
import org.eclipse.ui.part.PageBookView;

/**
 * This is a {@link PageBookView} for displaying some model {@link EObject} on
 * an {@link IPage}.
 * 
 * <p>
 * The model to show is determined by the following mechanism: <br>
 * It is expected that this view is opened via a call
 * <code>activePage.showView(viewID, editor.createIDForModel(model), IWorkbenchPage.VIEW_ACTIVATE);</code>
 * where
 * <ul>
 * <li> viewID is the ID which this view has been registered with in the
 * plugin.xml.
 * <li> model is the model to be displayed.
 * <li> editor is an instance of {@link AbstractTreeEditor} providing
 * createIDForModel(final EObject model).
 * </ul>
 * So this call sets an unique ID for the model as secondary ID of this view.
 * <br>
 * The active editor is being asked to resolve the ID to the model determined by
 * the previously set secondary ID.
 * </p>
 * 
 * <p>
 * The AbstractTreeEditor instance will hide this view when its model is being
 * deleted (i.e. eContainer is set to <code>null</code> somehow).
 * </p>
 * 
 * <p>
 * The user has to implement {@link #getPageForModel(EObject)} to create a page
 * (e.g. an {@link AbstractPageWithFlyoutPalette}) that is able to display or
 * even edit the model.
 * </p>
 * 
 * <p>
 * Additionally this view registers itself as an eAdapter of the model
 * {@link EObject} that can react on notifications of the EMF model. For this,
 * subclasses may implement
 * {@link Adapter#notifyChanged(org.eclipse.emf.common.notify.Notification)},
 * especially to update the name of this {@link IViewPart} according to the
 * model. {@link #calculatePartName()} must be implemented to represent e.g. the
 * model's name as a string.
 * </p>
 * 
 * <p>
 * This class is based roughly on ContentOutline but does not provide selections
 * e.g. to {@link ISelectionService}s.
 * </p>
 * 
 * @author Tony Modica
 */
public abstract class AbstractPageBookView extends PageBookView implements
		Adapter {

	/**
	 * The model to be displayed in the view's page.
	 */
	private EObject model;

	/**
	 * Target field for {@link Adapter} implementation.
	 */
	private Notifier target;

	/**
	 * @return the model
	 */
	public EObject getModel() {
		return model;
	}

	/**
	 * @return The name this part should be set to, e.g. the name of the model.
	 */
	abstract protected String calculatePartName();

	/**
	 * Subclasses must implement this method to create a {@link IPage} that
	 * displays the model. It may be advisable to ensure the model being an
	 * instance of the intended class(es) here.
	 * 
	 * @param model
	 *            The model to be displayed in the page.
	 * @return An {@link IPage} displaying the passed model.
	 */
	abstract protected IPage getPageForModel(EObject model);

	/*
	 * (non-Javadoc)
	 * 
	 * @see org.eclipse.ui.part.PageBookView#createDefaultPage(org.eclipse.ui.part.PageBook)
	 */
	@Override
	protected IPage createDefaultPage(final PageBook book) {
		final MessagePage page = new MessagePage();
		initPage(page);
		page.createControl(book);
		page
				.setMessage("The Editor did not return a model for ID: "
						+ getViewSite().getSecondaryId()
						+ "\nThis may happen if the file the previous editor was working on when this view was created can not be found, e.g. due to deletion.\nPlease close this view.");
		return page;
	}

	/**
	 * @return A PageRec with a new Page for the model determined by the
	 *         secondaryID which is being resolved via the editor.
	 * 
	 * @see #getPageForModel(Object)
	 */
	/*
	 * (non-Javadoc)
	 * 
	 * @see org.eclipse.ui.part.PageBookView#doCreatePage(org.eclipse.ui.IWorkbenchPart)
	 */
	@Override
	protected PageRec doCreatePage(final IWorkbenchPart editor) {
		// Die secondaryID des zu ffnenden Views lsst sich ber den Editor
		// in das Modellelement, das angezeigt werden soll, auflsen.
		final String secondaryID = getViewSite().getSecondaryId();
		Assert
				.isNotNull(
						secondaryID,
						"Secondary ID must be set for AbstractPageBookViews to determine a model to be shown!");

		model = ((AbstractTreeEditor) editor).getModelForID(secondaryID);

		// show default page if no model is returned by the editor
		if (null == model) {
			return null;
		}

		// register for listening for notifications on the model
		model.eAdapters().add(this);

		// create page for the model
		final IPage page = getPageForModel(model);
		initPage((IPageBookViewPage) page);
		page.createControl(getPageBook());

		// set the part's name for the first time
		setPartName(calculatePartName());

		return new PageRec(editor, page);
	}

	/*
	 * (non-Javadoc)
	 * 
	 * @see org.eclipse.ui.part.PageBookView#doDestroyPage(org.eclipse.ui.IWorkbenchPart,
	 *      org.eclipse.ui.part.PageBookView.PageRec)
	 */
	@Override
	protected void doDestroyPage(final IWorkbenchPart part, final PageRec rec) {
		model.eAdapters().remove(this);
		final IPage page = rec.page;
		page.dispose();
		rec.dispose();
	}

	/*
	 * (non-Javadoc)
	 * 
	 * @see org.eclipse.ui.part.PageBookView#getBootstrapPart()
	 */
	@Override
	protected IWorkbenchPart getBootstrapPart() {
		final IWorkbenchPage page = getSite().getPage();
		if (page != null) {
			return page.getActiveEditor();
		}
		return null;
	}

	/*
	 * (non-Javadoc)
	 * 
	 * @see org.eclipse.ui.part.PageBookView#isImportant(org.eclipse.ui.IWorkbenchPart)
	 */
	@Override
	protected boolean isImportant(final IWorkbenchPart part) {
		// We only care about editors
		return (part instanceof IEditorPart);
	}

	/**
	 * Treat this the same as part activation.
	 */
	/*
	 * (non-Javadoc)
	 * 
	 * @see org.eclipse.ui.IPartListener#partBroughtToTop(org.eclipse.ui.IWorkbenchPart)
	 */
	@Override
	public void partBroughtToTop(final IWorkbenchPart part) {
		partActivated(part);
	}

	/*
	 * (non-Javadoc)
	 * 
	 * @see org.eclipse.emf.common.notify.Adapter#isAdapterForType(java.lang.Object)
	 */
	public boolean isAdapterForType(final Object type) {
		return type.equals(model.getClass());
	}

	/*
	 * (non-Javadoc)
	 * 
	 * @see org.eclipse.emf.common.notify.Adapter#setTarget(org.eclipse.emf.common.notify.Notifier)
	 */
	public void setTarget(final Notifier newTarget) {
		target = newTarget;
	}

	/*
	 * (non-Javadoc)
	 * 
	 * @see org.eclipse.emf.common.notify.Adapter#getTarget()
	 */
	public Notifier getTarget() {
		return target;
	}
}
