package roneditor.model;

import java.io.IOException;
import java.util.HashMap;
import java.util.Map;

import org.eclipse.core.runtime.Assert;
import org.eclipse.core.runtime.IPath;
import org.eclipse.emf.common.util.URI;
import org.eclipse.emf.ecore.resource.Resource;
import org.eclipse.emf.ecore.resource.ResourceSet;
import org.eclipse.emf.ecore.resource.impl.ResourceSetImpl;
import org.eclipse.emf.ecore.xmi.XMLResource;
import org.eclipse.emf.ecore.xmi.impl.XMIResourceFactoryImpl;

import ronmodel.HLPlace;
import ronmodel.HLPlaceType;
import ronmodel.LHS;
import ronmodel.NAC;
import ronmodel.RHS;
import ronmodel.RON;
import ronmodel.RonmodelFactory;
import ronmodel.Rule;
import ronmodel.Token;
import ronmodel.impl.RonmodelPackageImpl;

/**
 * Manager for persistency operations on the model.
 * 
 * @author Tony Modica
 */
public class RONModelManager {

	/**
	 * For the purpose of the simple editor, a file can only contain one RON. In
	 * EMF, a Resource provides the way to have access to the model content.
	 */
	private Resource resource = null;

	/**
	 * Contains the factory associated with the model.
	 */
	private static RonmodelFactory ronmodelFactory = null;

	/**
	 * The top level RON in the resource.
	 */
	private RON ron = null;

	/**
	 * The ResourceSet
	 */
	private ResourceSet resourceSet = null;

	/**
	 * This constructor initializes the EMF model package and registers
	 */
	public RONModelManager() {
		// HINT Hier wird das unterliegende EMF-Package initialisiert
		RonmodelPackageImpl.init();
		// Register the XMI resource factory for the editor's file extension
		final Resource.Factory.Registry reg = Resource.Factory.Registry.INSTANCE;
		// HINT Hier muss die Dateiendung, auf die Euer Editor nach plugin.xml
		// reagiert angepasst werden
		reg.getExtensionToFactoryMap().put("ron", new XMIResourceFactoryImpl());
	}

	/**
	 * @param path
	 *            A {@link IPath} to a file containing a {@link Resource}.
	 * @return The resource containing the RON. This is initialized lazily with
	 *         the paramter <code>path</code>.
	 */
	public Resource getResource(final IPath path) {
		resource = getResourceSet().getResource(
				URI.createPlatformResourceURI(path.toString()), true);
		return resource;
	}

	/**
	 * @param path
	 *            A {@link IPath} to a file to be created for hosting the
	 *            {@link Resource}.
	 * @return The resource containing the RON. This is initialized lazily with
	 *         the parameter <code>path</code>.
	 */
	private Resource createResource(final IPath path) {
		if (resource == null) {
			resource = getResourceSet().createResource(
					URI.createPlatformResourceURI(path.toString()));
		}
		return resource;

	}

	/**
	 * @return The lazily created resourceSet.
	 */
	private ResourceSet getResourceSet() {
		if (null == resourceSet) {
			resourceSet = new ResourceSetImpl();
		}
		return resourceSet;
	}

	/**
	 * @return The factory associated with the model. Object are created using
	 *         that factory.
	 */
	static public RonmodelFactory getFactory() {
		if (ronmodelFactory == null) {
			ronmodelFactory = RonmodelFactory.eINSTANCE;
		}
		return ronmodelFactory;
	}

	/**
	 * Creates a new default RON in a Resource.
	 * 
	 * @param path
	 *            A {@link IPath} to a file to be created for hosting the
	 *            {@link Resource}.
	 * @return The newly created RON.
	 */
	public RON createDefaultRon(final IPath path) {
		// HINT Diese Methode erzeugt initial ein Modell fr leere (oder
		// beschdigte) an den Editor bergebene Dateien.
		ron = getFactory().createRON();
		createResource(path).getContents().add(ron);
		ron.setName("<default>");

		final HLPlace placeNET = getFactory().createHLPlace();
		placeNET.setName("<default>");
		placeNET.setType(HLPlaceType.NET_LITERAL);
		placeNET.setRon(ron);

		final Token netToken = getFactory().createObjectNet();
		netToken.setName("<default>");
		netToken.setPlace(placeNET);

		final HLPlace placeRULE = getFactory().createHLPlace();
		placeRULE.setName("<default>");
		placeRULE.setType(HLPlaceType.RULE_LITERAL);
		placeRULE.setRon(ron);

		// Rule with 3 NACs
		final Rule rule = getFactory().createRule();
		rule.setName("ruleWith3NACs");
		rule.setPlace(placeRULE);

		final LHS lhs = getFactory().createLHS();
		lhs.setName("lhs");
		lhs.setRule(rule);

		final RHS rhs = getFactory().createRHS();
		rhs.setName("rhs");
		rhs.setRule(rule);

		final NAC nac1 = getFactory().createNAC();
		nac1.setName("nac1");
		nac1.setRule(rule);

		final NAC nac2 = getFactory().createNAC();
		nac2.setName("nac2");
		nac2.setRule(rule);

		final NAC nac3 = getFactory().createNAC();
		nac3.setName("nac3");
		nac3.setRule(rule);

		// Rule with a single NAC
		final Rule rule2 = getFactory().createRule();
		rule2.setName("ruleWith1NAC");
		rule2.setPlace(placeRULE);

		final LHS lhs2 = getFactory().createLHS();
		lhs2.setName("lhs");
		lhs2.setRule(rule2);

		final RHS rhs2 = getFactory().createRHS();
		rhs2.setName("rhs");
		rhs2.setRule(rule2);

		final NAC singleNAC = getFactory().createNAC();
		singleNAC.setName("nac");
		singleNAC.setRule(rule2);

		return ron;
	}

	/**
	 * Loads the model from the file.
	 * 
	 * @param path
	 *            A {@link IPath} to a file containing a {@link Resource}.
	 * @throws IOException
	 */
	public void load(final IPath path) throws Exception {
		getResource(path);
		final Map<String, Boolean> options = new HashMap<String, Boolean>();
		options.put(XMLResource.OPTION_DECLARE_XML, Boolean.TRUE);
		try {
			resource.load(options);
		} catch (final Exception e) {
			// final StringBuffer buffer = new StringBuffer();
			// buffer.append(e.getLocalizedMessage());
			// buffer.append('\n');
			// final EList errors = this.resource.getErrors();
			// final EList warnings = this.resource.getWarnings();
			// if (!(errors.isEmpty() && warnings.isEmpty())) {
			// buffer.append("Found errors: ");
			// buffer.append(errors.toString());
			// buffer.append('\n');
			// buffer.append("Found warnings: ");
			// buffer.append(errors.toString());
			// // 2DO Informationen fuer Fehlerdiagnose anhaengen
			// handleLoadErrors(
			// "resource.load() in load() in RONModelManager",
			// buffer.toString());
			// }
			throw e;
		}

	}

	// private void handleLoadErrors(final String cause, final String message) {
	// // 2DO Fehlermeldungen ausgeben und ggf. zum Bugreport auffordern
	// System.out.println(cause);
	// System.out.println(message);
	// }

	/**
	 * Reloads the model from the file.
	 * 
	 * @param path
	 *            A {@link IPath} to a file containing a {@link Resource}.
	 * @throws IOException
	 */
	public void reload(final IPath path) throws Exception {
		getResource(path).unload();
		load(path);
	}

	/**
	 * Saves the content of the model to the file.
	 */
	public void save(final IPath path) throws IOException {
		// This sets the ron as contents in a new resource when using save as.
		getResource(path).getContents().add(ron);

		final Map<String, Boolean> options = new HashMap<String, Boolean>();
		options.put(XMLResource.OPTION_DECLARE_XML, Boolean.TRUE);
		resource.save(options);
	}

	/**
	 * @return The top level RON model from the resource.
	 */
	public RON getRON() {
		Assert.isNotNull(resource, "Resource has not been initialized!");
		if (null == ron) {
			for (final Object obj : resource.getContents()) {
				if (obj instanceof RON) {
					ron = (RON) obj;
				}
			}
		}
		return ron;
	}

}
