package roneditor.objectnetview;

import org.eclipse.emf.common.notify.Notification;
import org.eclipse.emf.ecore.EObject;
import org.eclipse.gef.EditPartFactory;
import org.eclipse.gef.EditPartViewer;
import org.eclipse.gef.KeyHandler;
import org.eclipse.gef.KeyStroke;
import org.eclipse.gef.palette.PaletteRoot;
import org.eclipse.gef.ui.actions.DeleteAction;
import org.eclipse.gef.ui.actions.PrintAction;
import org.eclipse.gef.ui.actions.SaveAction;
import org.eclipse.gef.ui.actions.SelectionAction;
import org.eclipse.swt.SWT;
import org.eclipse.ui.IEditorPart;

import roneditor.AbstractMultiViewerPage;
import roneditor.AbstractPageBookView;
import roneditor.actions.FireAction;
import roneditor.editor.ContextMenuProviderWithActionRegistry;
import roneditor.editor.RONEditor;
import roneditor.objectnetview.actions.EditAction;
import roneditor.objectnetview.actions.FireONTransitionAction;
import roneditor.objectnetview.actions.MoveNodeAction;
import roneditor.objectnetview.edit.utils.ObjectNetViewEditPartFactory;
import ronmodel.ONNode;
import ronmodel.ObjectNet;

/**
 * <p>
 * This is an implementation of {@link AbstractPageBookView} to display
 * {@link ObjectNet}s.
 * </p>
 * 
 * <p>
 * It uses {@link ObjectNetViewEditPartFactory} and
 * {@link ObjectNetPageContextMenuProvider}, configures the keyhandler and
 * creates actions to move {@link ONNode}s by pressing keys.
 * </p>
 * 
 * <p>
 * On model change nothing is done.
 * </p>
 * 
 * @author Tony Modica
 */
public class ObjectNetPage extends AbstractMultiViewerPage {

	public ObjectNetPage(final ObjectNet objectNet, final IEditorPart editor) {
		super(objectNet, editor);
	}

	/**
	 * @see roneditor.objectnetview.AbstractPageWithFlyoutPalette#createActions()
	 */
	@SuppressWarnings( { "deprecation", "deprecation" })
	@Override
	protected void createActions() {
		// HINT Hier werden Instanzen der Actions erzeugt und registriert,
		// damit der ContextMenuProvider sie in das Kontextmen einfgen kann.

		SelectionAction selectionAction;
		selectionAction = new MoveNodeAction(getEditor(), MoveNodeAction.LEFT);
		getActionRegistry().registerAction(selectionAction);

		selectionAction = new MoveNodeAction(getEditor(), MoveNodeAction.RIGHT);
		getActionRegistry().registerAction(selectionAction);

		selectionAction = new MoveNodeAction(getEditor(), MoveNodeAction.UP);
		getActionRegistry().registerAction(selectionAction);

		selectionAction = new MoveNodeAction(getEditor(), MoveNodeAction.DOWN);
		getActionRegistry().registerAction(selectionAction);

		selectionAction = new MoveNodeAction(getEditor(),
				MoveNodeAction.PREC_LEFT);
		getActionRegistry().registerAction(selectionAction);

		selectionAction = new MoveNodeAction(getEditor(),
				MoveNodeAction.PREC_RIGHT);
		getActionRegistry().registerAction(selectionAction);

		selectionAction = new MoveNodeAction(getEditor(),
				MoveNodeAction.PREC_UP);
		getActionRegistry().registerAction(selectionAction);

		selectionAction = new MoveNodeAction(getEditor(),
				MoveNodeAction.PREC_DOWN);
		getActionRegistry().registerAction(selectionAction);

		selectionAction = new DeleteAction(getEditor());
		getActionRegistry().registerAction(selectionAction);

		getActionRegistry().registerAction(new SaveAction(getEditor()));
		getActionRegistry().registerAction(new PrintAction(getEditor()));

		// Neue Action
		getActionRegistry().registerAction(new EditAction(getEditor()));
		getActionRegistry().registerAction(
				new FireONTransitionAction(getEditor()));
		getActionRegistry().registerAction(
				new FireAction((RONEditor) getEditor()));

	}

	/**
	 * @see roneditor.objectnetview.AbstractPageWithFlyoutPalette#setupKeyHandler(org.eclipse.gef.KeyHandler)
	 */
	@Override
	protected void setupKeyHandler(final KeyHandler kh) {
		// Arrow keys run the correspondent MoveNodeActions
		kh.put(KeyStroke.getPressed(SWT.ARROW_LEFT, 0), (getActionRegistry()
				.getAction(MoveNodeAction.LEFT)));
		kh.put(KeyStroke.getPressed(SWT.ARROW_RIGHT, 0), (getActionRegistry()
				.getAction(MoveNodeAction.RIGHT)));
		kh.put(KeyStroke.getPressed(SWT.ARROW_UP, 0), (getActionRegistry()
				.getAction(MoveNodeAction.UP)));
		kh.put(KeyStroke.getPressed(SWT.ARROW_DOWN, 0), (getActionRegistry()
				.getAction(MoveNodeAction.DOWN)));

		// SHIFT + Arrow keys run the correspondent precise MoveNodeActions
		kh.put(KeyStroke.getPressed(SWT.ARROW_LEFT, SWT.SHIFT),
				(getActionRegistry().getAction(MoveNodeAction.PREC_LEFT)));
		kh.put(KeyStroke.getPressed(SWT.ARROW_RIGHT, SWT.SHIFT),
				(getActionRegistry().getAction(MoveNodeAction.PREC_RIGHT)));
		kh.put(KeyStroke.getPressed(SWT.ARROW_UP, SWT.SHIFT),
				(getActionRegistry().getAction(MoveNodeAction.PREC_UP)));
		kh.put(KeyStroke.getPressed(SWT.ARROW_DOWN, SWT.SHIFT),
				(getActionRegistry().getAction(MoveNodeAction.PREC_DOWN)));
	}

	/**
	 * @see roneditor.objectnetview.AbstractPageWithFlyoutPalette#createEditPartFactory()
	 */
	@Override
	protected EditPartFactory createEditPartFactory() {
		return new ObjectNetViewEditPartFactory();
	}

	/**
	 * 
	 * @see roneditor.objectnetview.AbstractPageWithFlyoutPalette#getContextMenuProvider()
	 */
	@Override
	protected ContextMenuProviderWithActionRegistry createContextMenuProvider(
			final EditPartViewer viewer) {
		return new ObjectNetPageContextMenuProvider(viewer, getActionRegistry());
	}

	/**
	 * @see roneditor.objectnetview.AbstractPageWithFlyoutPalette#createPaletteRoot()
	 */
	@Override
	protected PaletteRoot createPaletteRoot() {
		// just an empty palette root
		return new ObjectNetPaletteRoot();
	}

	/**
	 * @see roneditor.objectnetview.AbstractPageWithFlyoutPalette#getThumbSashWeights()
	 */
	@Override
	protected int[] getThumbSashWeights() {
		return new int[] { 3, 1 };
	}

	/**
	 * @see org.eclipse.emf.common.notify.Adapter#notifyChanged(org.eclipse.emf.common.notify.Notification)
	 */
	public void notifyChanged(final Notification notification) {
		// the page doesn't care for model changes (yet)
	}

	/**
	 * 
	 * @see roneditor.ruleview.AbstractMultiViewerPage#getNumberOfViewers()
	 */
	@Override
	public int getNumberOfViewers() {
		return 1;
	}

	/**
	 * 
	 * @see roneditor.ruleview.AbstractMultiViewerPage#getViewerContents()
	 */
	@Override
	public EObject[] getViewerContents() {
		return new EObject[] { getModel() };
	}
	/**
	 * 
	 * @return the ObjectNet
	 */
	public ObjectNet getObjectNet() {
		return (ObjectNet) getModel();
	}
}
