package roneditor.objectnetview.commands;

import java.util.Iterator;
import java.util.Vector;

import org.eclipse.emf.ecore.EObject;

import ronmodel.Codomain;
import ronmodel.Mapping;
import ronmodel.ONArc;
import ronmodel.ONNode;
import ronmodel.ONPlace;
import ronmodel.ONTransition;
import ronmodel.ObjectNet;

/**
 * Command for deleting a transition.
 */
public class DeleteONNodeCommand extends org.eclipse.gef.commands.Command {
	private static final String DeleteCommand_Label = "DeleteONNodeCommand";

	private ONNode onNode;

	private int x;

	private int y;

	private ObjectNet objectNet;

	private Vector<EObject> incomingONArcs = new Vector<EObject>();

	private Vector<EObject> outgoingONArcs = new Vector<EObject>();

	private Vector<EObject> incomingMappings = new Vector<EObject>();

	private Vector<EObject> outgoingMappings = new Vector<EObject>();

	/**
	 * 
	 */
	public DeleteONNodeCommand() {
		super(DeleteCommand_Label);
	}

	/**
	 * Delete all arcs which are connected with the transition.
	 * 
	 * @param onNode
	 */
	private void deleteArcs(ONNode onNode) {
		incomingONArcs.clear();
		outgoingONArcs.clear();
		for (Iterator it = onNode.getIncoming().iterator(); it.hasNext();) {
			ONArc onArc = (ONArc) it.next();
			ONNode source = onArc.getSource();
			incomingONArcs.add(onArc);
			incomingONArcs.add(source);
		}
		for (int i = 0; i < incomingONArcs.size(); i = i + 2) {
			ONArc onArc = (ONArc) incomingONArcs.get(i);
			onArc.setSource(null);
			onArc.setTarget(null);
			onArc.setObjectNet(null);
		}

		for (Iterator it = onNode.getOutgoing().iterator(); it.hasNext();) {
			ONArc onArc = (ONArc) it.next();
			ONNode target = onArc.getTarget();
			outgoingONArcs.add(onArc);
			outgoingONArcs.add(target);
		}
		for (int i = 0; i < outgoingONArcs.size(); i = i + 2) {
			ONArc onArc = (ONArc) outgoingONArcs.get(i);
			onArc.setSource(null);
			onArc.setTarget(null);
			onArc.setObjectNet(null);
		}
	}

	/**
	 * Delete all mappings which are connected with another node.
	 * 
	 * @param onNode
	 */
	private void deleteMappings(ONNode onNode) {
		incomingMappings.clear();
		outgoingMappings.clear();
		for (Iterator it = onNode.getIncomingMappings().iterator(); it
				.hasNext();) {
			Mapping mapping = (Mapping) it.next();
			ONNode source = mapping.getSource();
			incomingMappings.add(mapping);
			incomingMappings.add(source);
		}
		for (int i = 0; i < incomingMappings.size(); i = i + 2) {
			Mapping mapping = (Mapping) incomingMappings.get(i);
			mapping.setSource(null);
			mapping.setTarget(null);
			mapping.setCodomain(null);

		}
		for (Iterator it = onNode.getOutgoingMappings().iterator(); it
				.hasNext();) {
			Mapping mapping = (Mapping) it.next();
			ONNode target = mapping.getTarget();
			target.setMappingID(0);
			outgoingMappings.add(mapping);
			outgoingMappings.add(target);
		}
		for (int i = 0; i < outgoingMappings.size(); i = i + 2) {
			Mapping mapping = (Mapping) outgoingMappings.get(i);
			mapping.setSource(null);
			mapping.setTarget(null);
			mapping.setCodomain(null);

		}
	}

	/**
	 * Entry point for command execution.
	 * ein {@link ONNode} wird aus einem {@link ObjectNet} gelscht.
	 * dabei werden alle {@link ONArc onArcs} und {@link Mapping mappings} 
	 * von diesem {@link ONNode} mitgelscht 
	 */
	public void execute() {
		deleteMappings(onNode);
		deleteArcs(onNode);
		onNode.setObjectNet(null);
	}

	public void redo() {
		execute();
	}

	/**
	 * Undo command execution.
	 */
	public void undo() {
		for (int i = 0; i < incomingONArcs.size(); i = i + 2) {
			ONArc onArc = (ONArc) incomingONArcs.get(i);
			ONNode source = (ONNode) incomingONArcs.get(i + 1);
			onArc.setSource(source);
			onArc.setTarget(onNode);
			onArc.setObjectNet(objectNet);
		}
		for (int i = 0; i < outgoingONArcs.size(); i = i + 2) {
			ONArc onArc = (ONArc) outgoingONArcs.get(i);
			ONNode target = (ONNode) outgoingONArcs.get(i + 1);
			onArc.setSource(onNode);
			onArc.setTarget(target);
			onArc.setObjectNet(objectNet);
		}
		for (int i = 0; i < incomingMappings.size(); i = i + 2) {
			Mapping mapping = (Mapping) incomingMappings.get(i);
			ONNode source = (ONNode) incomingMappings.get(i + 1);
			mapping.setSource(source);
			mapping.setTarget(onNode);
			mapping.setCodomain((Codomain) objectNet);
		}
		for (int i = 0; i < outgoingMappings.size(); i = i + 2) {
			Mapping mapping = (Mapping) outgoingMappings.get(i);
			ONNode target = (ONNode) outgoingMappings.get(i + 1);
			mapping.setSource(onNode);
			mapping.setTarget(target);
			target.setMappingID(onNode.getMappingID());
			mapping.setCodomain((Codomain) target.getObjectNet());
		}
		onNode.setObjectNet(objectNet);
	}

	@Override
	public boolean canExecute() {
		if (onNode == null || objectNet == null) {
			return false;
		}
		return super.canExecute();
	}

	/**
	 * @return the objectNet
	 */
	public ObjectNet getObjectNet() {
		return objectNet;
	}

	/**
	 * @param objectNet
	 *            the objectNet to set
	 */
	public void setObjectNet(ObjectNet objectNet) {
		this.objectNet = objectNet;
	}

	/**
	 * @return the node
	 */
	public ONNode getONNode() {
		return onNode;
	}

	/**
	 * @param node
	 *            the node to set
	 */
	public void setONNode(ONNode node) {
		this.onNode = node;
		if (node instanceof ONPlace) {
			setLabel("delete place");
		} else if (node instanceof ONTransition) {
			setLabel("delete transition");
		}
	}

	/**
	 * @return the x
	 */
	public int getX() {
		return x;
	}

	/**
	 * @param x
	 *            the x to set
	 */
	public void setX(int x) {
		this.x = x;
	}

	/**
	 * @return the y
	 */
	public int getY() {
		return y;
	}

	/**
	 * @param y
	 *            the y to set
	 */
	public void setY(int y) {
		this.y = y;
	}

}
