package roneditor.ruleview;

import org.eclipse.emf.common.notify.Notification;
import org.eclipse.emf.ecore.EObject;
import org.eclipse.gef.EditPartFactory;
import org.eclipse.gef.EditPartViewer;
import org.eclipse.gef.KeyHandler;
import org.eclipse.gef.KeyStroke;
import org.eclipse.gef.palette.PaletteRoot;
import org.eclipse.gef.ui.actions.DeleteAction;
import org.eclipse.gef.ui.actions.SelectionAction;
import org.eclipse.jface.action.Action;
import org.eclipse.swt.SWT;
import org.eclipse.ui.IEditorPart;

import roneditor.AbstractMultiViewerPage;
import roneditor.AbstractPageBookView;
import roneditor.editor.ContextMenuProviderWithActionRegistry;
import roneditor.objectnetview.actions.EditAction;
import roneditor.objectnetview.actions.MoveNodeAction;
import roneditor.objectnetview.edit.utils.ObjectNetViewEditPartFactory;
import roneditor.ruleview.actions.CopyLHS2NACAction;
import roneditor.ruleview.actions.CopyLHS2RHSAction;
import roneditor.ruleview.actions.CopyRHS2NACAction;
import roneditor.ruleview.actions.CreateNACAction;
import roneditor.ruleview.actions.DeleteMappingAction;
import roneditor.ruleview.actions.DeleteNACAction;
import roneditor.ruleview.actions.SwitchNACAction;
import roneditor.ruleview.edit.utils.RuleViewEditPartFactory;
import ronmodel.LHS;
import ronmodel.NAC;
import ronmodel.ONNode;
import ronmodel.RHS;
import ronmodel.Rule;

/**
 * <p>
 * This is an implementation of {@link AbstractPageBookView} to display
 * {@link Rule}s.
 * </p>
 * 
 * <p>
 * It uses {@link ObjectNetViewEditPartFactory} and
 * {@link RulePageContextMenuProvider}, configures the keyhandler and creates
 * actions to move {@link ONNode}s by pressing keys.
 * </p>
 * 
 * <p>
 * On model change nothing is done.
 * </p>
 * 
 * @author Tony Modica
 */
public class RulePage extends AbstractMultiViewerPage {

	public RulePage(final Rule rule, final IEditorPart editor) {
		super(rule, editor);
	}

	/*
	 * (non-Javadoc)
	 * 
	 * @see roneditor.objectnetview.AbstractPageWithFlyoutPalette#createActions()
	 */
	@Override
	protected void createActions() {
		// HINT Hier werden Instanzen der Actions erzeugt und registriert,
		// damit der ContextMenuProvider sie in das Kontextmen einfgen kann.

		SelectionAction selectionAction;
		selectionAction = new MoveNodeAction(getEditor(), MoveNodeAction.LEFT);
		getActionRegistry().registerAction(selectionAction);

		selectionAction = new MoveNodeAction(getEditor(), MoveNodeAction.RIGHT);
		getActionRegistry().registerAction(selectionAction);

		selectionAction = new MoveNodeAction(getEditor(), MoveNodeAction.UP);
		getActionRegistry().registerAction(selectionAction);

		selectionAction = new MoveNodeAction(getEditor(), MoveNodeAction.DOWN);
		getActionRegistry().registerAction(selectionAction);

		selectionAction = new MoveNodeAction(getEditor(),
				MoveNodeAction.PREC_LEFT);
		getActionRegistry().registerAction(selectionAction);

		selectionAction = new MoveNodeAction(getEditor(),
				MoveNodeAction.PREC_RIGHT);
		getActionRegistry().registerAction(selectionAction);

		selectionAction = new MoveNodeAction(getEditor(),
				MoveNodeAction.PREC_UP);
		getActionRegistry().registerAction(selectionAction);

		selectionAction = new MoveNodeAction(getEditor(),
				MoveNodeAction.PREC_DOWN);
		getActionRegistry().registerAction(selectionAction);

		selectionAction = new DeleteAction(getEditor());
		getActionRegistry().registerAction(selectionAction);
		// A sample action to switch through the NACs
		final Action action = new SwitchNACAction(getEditor(), this);
		getActionRegistry().registerAction(action);

		getActionRegistry().registerAction(new EditAction(getEditor()));
		getActionRegistry().registerAction(
				new CopyLHS2RHSAction(getEditor(), this));
		getActionRegistry().registerAction(
				new CopyLHS2NACAction(getEditor(), this));
		getActionRegistry().registerAction(
				new CopyRHS2NACAction(getEditor(), this));
		getActionRegistry().registerAction(
				new CreateNACAction(getEditor(), this));
		getActionRegistry().registerAction(
				new DeleteNACAction(getEditor(), this));
		getActionRegistry()
				.registerAction(new DeleteMappingAction(getEditor()));
	}

	/*
	 * (non-Javadoc)
	 * 
	 * @see roneditor.objectnetview.AbstractPageWithFlyoutPalette#setupKeyHandler(org.eclipse.gef.KeyHandler)
	 */
	@Override
	protected void setupKeyHandler(final KeyHandler kh) {
		// Arrow keys run the correspondent MoveNodeActions
		kh.put(KeyStroke.getPressed(SWT.ARROW_LEFT, 0), (getActionRegistry()
				.getAction(MoveNodeAction.LEFT)));
		kh.put(KeyStroke.getPressed(SWT.ARROW_RIGHT, 0), (getActionRegistry()
				.getAction(MoveNodeAction.RIGHT)));
		kh.put(KeyStroke.getPressed(SWT.ARROW_UP, 0), (getActionRegistry()
				.getAction(MoveNodeAction.UP)));
		kh.put(KeyStroke.getPressed(SWT.ARROW_DOWN, 0), (getActionRegistry()
				.getAction(MoveNodeAction.DOWN)));

		// SHIFT + Arrow keys run the correspondent precise MoveNodeActions
		kh.put(KeyStroke.getPressed(SWT.ARROW_LEFT, SWT.SHIFT),
				(getActionRegistry().getAction(MoveNodeAction.PREC_LEFT)));
		kh.put(KeyStroke.getPressed(SWT.ARROW_RIGHT, SWT.SHIFT),
				(getActionRegistry().getAction(MoveNodeAction.PREC_RIGHT)));
		kh.put(KeyStroke.getPressed(SWT.ARROW_UP, SWT.SHIFT),
				(getActionRegistry().getAction(MoveNodeAction.PREC_UP)));
		kh.put(KeyStroke.getPressed(SWT.ARROW_DOWN, SWT.SHIFT),
				(getActionRegistry().getAction(MoveNodeAction.PREC_DOWN)));
	}

	/*
	 * (non-Javadoc)
	 * 
	 * @see roneditor.objectnetview.AbstractPageWithFlyoutPalette#createEditPartFactory()
	 */
	@Override
	protected EditPartFactory createEditPartFactory() {
		// Just show the nets like in ObjectNetView, nothing special here
		return new RuleViewEditPartFactory();
	}

	/*
	 * (non-Javadoc)
	 * 
	 * @see roneditor.objectnetview.AbstractPageWithFlyoutPalette#getContextMenuProvider()
	 */
	@Override
	protected ContextMenuProviderWithActionRegistry createContextMenuProvider(
			final EditPartViewer viewer) {
		return new RulePageContextMenuProvider(viewer, getActionRegistry());
	}

	/*
	 * (non-Javadoc)
	 * 
	 * @see roneditor.objectnetview.AbstractPageWithFlyoutPalette#createPaletteRoot()
	 */
	@Override
	protected PaletteRoot createPaletteRoot() {
		// just an empty palette root
		return new RulePaletteRoot();
	}

	/*
	 * (non-Javadoc)
	 * 
	 * @see roneditor.objectnetview.AbstractPageWithFlyoutPalette#getThumbSashWeights()
	 */
	@Override
	protected int[] getThumbSashWeights() {
		return new int[] { 3, 1 };
	}

	/*
	 * (non-Javadoc)
	 * 
	 * @see org.eclipse.emf.common.notify.Adapter#notifyChanged(org.eclipse.emf.common.notify.Notification)
	 */
	public void notifyChanged(final Notification notification) {
		int type = notification.getEventType();
		if (type == Notification.ADD
				&& notification.getNewValue() instanceof NAC) {
			if (getRule().getNacs().size() == 1) {
				switchViewerVisibility(0);
			}
			NAC nac = (NAC) notification.getNewValue();
			setViewersContents(0, nac);
		}
		if (type == Notification.REMOVE
				&& notification.getOldValue() instanceof NAC) {
			if (getRule().getNacs().isEmpty()) {
				switchViewerVisibility(0);
			} else {
				setViewersContents(0, (EObject) getRule().getNacs().get(0));
			}
		}
	}

	/**
	 * A {@link RulePage} shows three nets: {@link NAC}, {@link LHS} and
	 * {@link RHS} of the {@link Rule}.
	 */
	/*
	 * (non-Javadoc)
	 * 
	 * @see roneditor.AbstractMultiViewerPage#getNumberOfViewers()
	 */
	@Override
	public int getNumberOfViewers() {
		final Rule rule = getRule();
		return 3;
	}

	/**
	 * A {@link RulePage} shows the first {@link NAC} of the {@link Rule} and
	 * the {@link LHS} and {@link RHS}.
	 */
	/*
	 * (non-Javadoc)
	 * 
	 * @see roneditor.AbstractMultiViewerPage#getViewerContents()
	 */
	@Override
	public EObject[] getViewerContents() {
		final Rule rule = getRule();
		return new EObject[] { (EObject) rule.getNacs().get(0), rule.getLhs(),
				rule.getRhs() };
	}

	/**
	 * @return The model of the page casted to a {@link Rule}.
	 */
	public Rule getRule() {
		return (Rule) getModel();
	}
}
